/*	$NetBSD$	*/

/*-
 * Copyright (c) 2010 SHIMIZU Ryo <ryo@nerv.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/cdefs.h>
__KERNEL_RCSID(0, "$NetBSD$");

#include <sys/param.h>
#include <sys/device.h>

#include <machine/bus.h>

/*
 * Terminology for reading source and PDF
 *	IPU: Image Processing Unit
 *	VDI: Video De-Interlacer
 *	CSI: Camera Sensor Interface
 *	CM: Control Module
 *	DI: Display Interface
 *	DP: Display Processor
 *	DC: Display Controller
 *	IC: Image Converter
 *	SMFC: Sensor Multi FIFO Control
 *	DMFC: Display Multi Fifo Control
 *	IDMAC: Image DMA Controller
 *	IRT: Image Rotator
 */


#include <arm/imx/imx51reg.h>
#include <arm/imx/imx51var.h>
#include <arm/imx/imx51_ipureg.h>
#include <arm/imx/imx51_iomuxreg.h>
#include <sys/gpio.h>

struct ipu_softc {
	device_t sc_dev;
	struct axi_softc *sc_axi;
	bus_addr_t sc_addr;
	bus_space_tag_t sc_iot;
	bus_space_handle_t sc_ioh_framebuffer;	/* +0x00000000 - 0x0E000000 */
	bus_space_handle_t sc_ioh_common;	/* +0x0E000000 - 0x0E070000 */
	bus_space_handle_t sc_ioh_cpmem;	/* +0x0F000000 - 0x0F020000 */
	bus_space_handle_t sc_ioh_lut;		/* +0x0F020000 - 0x0F040000 */
	bus_space_handle_t sc_ioh_srm;		/* +0x0F040000 - 0x0F060000 */
	bus_space_handle_t sc_ioh_tpm;		/* +0x0F060000 - 0x0F080000 */
	bus_space_handle_t sc_ioh_dctemp;	/* +0x0F080000 - 0x0F0a0000 */
	void *sc_ih_err;
	void *sc_ih_sync;
	bus_size_t sc_size;
};

static int ipu_match(device_t, struct cfdata *, void *);
static void ipu_attach(device_t, device_t, void *);
static void ipu_init(struct ipu_softc *);
static void ipu_lcd_init(struct ipu_softc *);
static void ipu_lcd_power(struct ipu_softc *, int);
static int ipu_err_intr(void *);
static int ipu_sync_intr(void *);

CFATTACH_DECL_NEW(imx51ipu, sizeof(struct ipu_softc),
    ipu_match, ipu_attach, NULL, NULL);

/* ARGSUSED */
static int
ipu_match(device_t parent __unused, struct cfdata *match __unused,
    void *aux)
{
	struct axi_attach_args *aa;

	aa = aux;

	switch (aa->aa_addr) {
	case IMX51_IPU_BASE:
		break;
	default:
		return 0;
	}

	aa->aa_size = IMX51_IPU_SIZE;

	return 1;
}

static void
ipu_attach(device_t parent, device_t self, void *aux)
{
	struct ipu_softc *sc;
	struct axi_attach_args *aa;

	aprint_normal(": Image Processing Unit\n");
	aprint_naive("\n");

	aa = aux;
	sc = device_private(self);
	sc->sc_axi = device_private(parent);
	sc->sc_dev = self;
	sc->sc_iot = aa->aa_iot;
	sc->sc_addr = aa->aa_addr;
	sc->sc_size = aa->aa_size;

	if (bus_space_map(sc->sc_iot, sc->sc_addr + IMX51_IPU_FRBUFFER_OFFSET,
	    IMX51_IPU_FRBUFFER_SIZE, 0, &sc->sc_ioh_framebuffer))
	    panic("%s: Cannot map framebuffer", device_xname(self));
	if (bus_space_map(sc->sc_iot, sc->sc_addr + IMX51_IPU_COMMON_OFFSET,
	    IMX51_IPU_COMMON_SIZE, 0, &sc->sc_ioh_common))
	    panic("%s: Cannot map common registers", device_xname(self));
	if (bus_space_map(sc->sc_iot, sc->sc_addr+ IMX51_IPU_LUT_OFFSET,
	    IMX51_IPU_LUT_SIZE, 0, &sc->sc_ioh_lut))
	    panic("%s: Cannot map LUT", device_xname(self));
	if (bus_space_map(sc->sc_iot, sc->sc_addr+ IMX51_IPU_SRM_OFFSET,
	    IMX51_IPU_SRM_SIZE, 0, &sc->sc_ioh_srm))
	    panic("%s: Cannot map SRM", device_xname(self));
	if (bus_space_map(sc->sc_iot, sc->sc_addr+ IMX51_IPU_TPM_OFFSET,
	    IMX51_IPU_TPM_SIZE, 0, &sc->sc_ioh_tpm))
	    panic("%s: Cannot map TPM", device_xname(self));
	if (bus_space_map(sc->sc_iot, sc->sc_addr+ IMX51_IPU_DCTEMP_OFFSET,
	    IMX51_IPU_DCTEMP_SIZE, 0, &sc->sc_ioh_dctemp))
	    panic("%s: Cannot map DC template", device_xname(self));

	sc->sc_ih_err = intr_establish(aa->aa_irq, IPL_CLOCK, 0,
	    ipu_err_intr, sc);
	if (sc->sc_ih_err == NULL)
		aprint_error_dev(self, "unable to register error interrupt\n");
	sc->sc_ih_sync = intr_establish(aa->aa_irq + 1, IPL_CLOCK, 0,
	    ipu_sync_intr, sc);
	if (sc->sc_ih_sync == NULL)
		aprint_error_dev(self, "unable to register sync interrupt\n");

	/* initialize */
	ipu_init(sc);

	ipu_lcd_init(sc);
	ipu_lcd_power(sc, 1);
}

static void
ipu_init(struct ipu_softc *sc)
{
	uint32_t v;

	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common,
	    IMX51_IPU_COMMON_IPU_MEM_RST_REG, 0x807fffff);

	while (bus_space_read_4(sc->sc_iot, sc->sc_ioh_common,
	    IMX51_IPU_COMMON_IPU_MEM_RST_REG) &
	    IMX51_IPU_COMMON_IPU_MEM_RST_START)
		;

	/* enable interrupts */
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common,
	    IMX51_IPU_COMMON_IPU_INT_CTRL_5_REG, 0xffffffff);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common,
	    IMX51_IPU_COMMON_IPU_INT_CTRL_6_REG, 0xffffffff);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common,
	    IMX51_IPU_COMMON_IPU_INT_CTRL_9_REG, 0xffffffff);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common,
	    IMX51_IPU_COMMON_IPU_INT_CTRL_10_REG, 0xffffffff);


	/* configure mapping registers */
	v = bus_space_read_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_0_REG);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_0_REG, v & 0xffff0000);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_0_REG, 0x000007ff);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_15_REG, 0x00000000);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_0_REG, 0x0fff07ff);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_15_REG, 0x00000020);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_0_REG, 0x000017ff);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_15_REG, 0x00000820);

	v = bus_space_read_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_1_REG);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_1_REG, v & 0x0000ffff);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_1_REG, 0x05fc17ff);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_16_REG, 0x00030820);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_1_REG, 0x00000bfc);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_16_REG, 0x00830820);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_1_REG, 0x11fc0bfc);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_16_REG, 0x14830820);

	v = bus_space_read_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_2_REG);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_2_REG, v & 0xffff0000);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_2_REG, 0x00000fff);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_17_REG, 0x00000006);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_2_REG, 0x17ff0fff);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_17_REG, 0x000000e6);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_2_REG, 0x000007ff);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_17_REG, 0x000020e6);

	v = bus_space_read_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_3_REG);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_3_REG, v & 0x0000ffff);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_3_REG, 0x04f807ff);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_18_REG, 0x000920e6);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_3_REG, 0x00000afc);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_18_REG, 0x014920e6);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_3_REG, 0x0ff80afc);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_18_REG, 0x2d4920e6);

	v = bus_space_read_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_4_REG);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_4_REG, v & 0xffff0000);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_4_REG, 0x000005fc);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_19_REG, 0x0000000c);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_4_REG, 0x0dfc05fc);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_19_REG, 0x000001ac);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_4_REG, 0x000015fc);
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_DC_MAP_CONF_19_REG, 0x000039ac);



	/* DMFC */
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common,
	    IMX51_IPU_COMMON_DMFC_IC_CTRL_REG,
	    IMX51_IPU_COMMON_DMFC_IC_CTRL_IN_PORT(2));

	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common,
	    IMX51_IPU_COMMON_DMFC_WR_CHAN_ALT_REG, 0x88);		/* XXX: unknown from linux */
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common,
	    IMX51_IPU_COMMON_DMFC_WR_CHAN_DEF_ALT_REG, 0x202020F6);	/* XXX: unknown from linux */
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common,
	    IMX51_IPU_COMMON_DMFC_DP_CHAN_ALT_REG, 0x1F1E9694);		/* XXX: unknown from linux */
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common,
	    IMX51_IPU_COMMON_DMFC_DP_CHAN_DEF_ALT_REG, 0x2020F6F6);	/* XXX: unknown from linux */


	/* sync refresh */
	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common,
	    IMX51_IPU_COMMON_IDMAC_CH_PRI_1_REG, 0x18800000);

	bus_space_write_4(sc->sc_iot, sc->sc_ioh_common, IMX51_IPU_COMMON_IPU_DISP_GEN_REG,
	    IMX51_IPU_COMMON_IPU_DISP_GEN_MCU_MAX_BURST_STOP |
	    IMX51_IPU_COMMON_IPU_DISP_GEN_MCU_T(8));


}


static void
ipu_lcd_init(struct ipu_softc *sc)
{
	/* DISPLAY1 */
	AXI_IOMUXC_MUX(sc->sc_axi, DI1_PIN2, ALT0);	/* DI1_PIN2 HSYNC */
	AXI_IOMUXC_MUX(sc->sc_axi, DI1_PIN3, ALT0);	/* DI1_PIN3 VSYNC */

	AXI_IOMUXC_MUX(sc->sc_axi, DISP1_DAT0, ALT0);	/* DISP1_DAT0 (fixed) */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP1_DAT1, ALT0);	/* DISP1_DAT1 (fixed) */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP1_DAT2, ALT0);	/* DISP1_DAT2 (fixed) */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP1_DAT3, ALT0);	/* DISP1_DAT3 (fixed) */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP1_DAT4, ALT0);	/* DISP1_DAT4 (fixed) */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP1_DAT5, ALT0);	/* DISP1_DAT5 (fixed) */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP1_DAT6, ALT0);	/* DISP1_DAT6 */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP1_DAT7, ALT0);	/* DISP1_DAT7 */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP1_DAT8, ALT0);	/* DISP1_DAT8 */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP1_DAT9, ALT0);	/* DISP1_DAT9 */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP1_DAT10, ALT0);	/* DISP1_DAT10 */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP1_DAT11, ALT0);	/* DISP1_DAT11 */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP1_DAT12, ALT0);	/* DISP1_DAT12 */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP1_DAT13, ALT0);	/* DISP1_DAT13 */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP1_DAT14, ALT0);	/* DISP1_DAT14 */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP1_DAT15, ALT0);	/* DISP1_DAT15 */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP1_DAT16, ALT0);	/* DISP1_DAT16 */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP1_DAT17, ALT0);	/* DISP1_DAT17 */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP1_DAT18, ALT0);	/* DISP1_DAT18 */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP1_DAT19, ALT0);	/* DISP1_DAT19 */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP1_DAT20, ALT0);	/* DISP1_DAT20 */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP1_DAT21, ALT0);	/* DISP1_DAT21 */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP1_DAT22, ALT0);	/* DISP1_DAT22 */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP1_DAT23, ALT0);	/* DISP1_DAT23 */

	/* DISPLAY2 */
	AXI_IOMUXC_MUX(sc->sc_axi, DI_GP4, ALT4);	/* DI2_PIN15 DRDY */
	AXI_IOMUXC_MUX(sc->sc_axi, DI2_DISP_CLK, ALT0);	/* DI2_DISP_CLK */

	AXI_IOMUXC_MUX(sc->sc_axi, DI2_PIN2, ALT0);	/* DI2_PIN2 HSYNC */
	AXI_IOMUXC_MUX(sc->sc_axi, DI2_PIN3, ALT0);	/* DI2_PIN3 VSYNC */

	AXI_IOMUXC_MUX(sc->sc_axi, DISP2_DAT0, ALT0);	/* DISP2 DAT[0] */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP2_DAT1, ALT0);	/* DISP2 DAT[1] */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP2_DAT2, ALT0);	/* DISP2 DAT[2] */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP2_DAT3, ALT0);	/* DISP2 DAT[3] */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP2_DAT4, ALT0);	/* DISP2 DAT[4] */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP2_DAT5, ALT0);	/* DISP2 DAT[5] */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP2_DAT6, ALT0);	/* DISP2 DAT[6] */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP2_DAT7, ALT0);	/* DISP2 DAT[7] */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP2_DAT8, ALT0);	/* DISP2 DAT[8] */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP2_DAT9, ALT0);	/* DISP2 DAT[9] */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP2_DAT10, ALT0);	/* DISP2 DAT[10] */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP2_DAT11, ALT0);	/* DISP2 DAT[11] */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP2_DAT12, ALT0);	/* DISP2 DAT[12] */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP2_DAT13, ALT0);	/* DISP2 DAT[13] */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP2_DAT14, ALT0);	/* DISP2 DAT[14] */
	AXI_IOMUXC_MUX(sc->sc_axi, DISP2_DAT15, ALT0);	/* DISP2 DAT[15] */


#ifdef NETWALKER
	/*
	 * NetWalker specific?
	 */

	/* GPIO2-13 EIM_A19 */
	AXI_IOMUXC_MUXPAD(sc->sc_axi, EIM_A19, ALT1,
	    IMX51_IOMUXC_SW_PAD_CTL_PAD_HYS);
	axi_gpio_dir(sc->sc_axi, AXI_GPIO2, 13, GPIO_PIN_OUTPUT);
	axi_gpio_write(sc->sc_axi, AXI_GPIO2, 13, 1);

	/* LCD LVDS */
	AXI_IOMUXC_MUXPAD(sc->sc_axi, DI1_D0_CS, ALT4,
	    IMX51_IOMUXC_SW_PAD_CTL_PAD_PKE |
	    IMX51_IOMUXC_SW_PAD_CTL_PAD_DS_HIGH |
	    IMX51_IOMUXC_SW_PAD_CTL_PAD_SR_FAST);
	axi_gpio_dir(sc->sc_axi, AXI_GPIO3, 3, GPIO_PIN_OUTPUT);

	/* LCD 3.3V */
	AXI_IOMUXC_MUXPAD(sc->sc_axi, CSI2_D12, ALT3,
	    IMX51_IOMUXC_SW_PAD_CTL_PAD_PKE |
	    IMX51_IOMUXC_SW_PAD_CTL_PAD_DS_HIGH |
	    IMX51_IOMUXC_SW_PAD_CTL_PAD_SR_FAST);
	axi_gpio_dir(sc->sc_axi, AXI_GPIO4, 9, GPIO_PIN_OUTPUT);

	/* LCD 5V */
	AXI_IOMUXC_MUXPAD(sc->sc_axi, CSI2_D13, ALT3,
	    IMX51_IOMUXC_SW_PAD_CTL_PAD_PKE |
	    IMX51_IOMUXC_SW_PAD_CTL_PAD_DS_HIGH |
	    IMX51_IOMUXC_SW_PAD_CTL_PAD_SR_FAST);
	axi_gpio_dir(sc->sc_axi, AXI_GPIO4, 10, GPIO_PIN_OUTPUT);
#endif /* NETWALKER */


}

static void
ipu_lcd_power(struct ipu_softc *sc, int on)
{
#ifdef NETWALKER
	/*
	 * Netwalker specific?
	 */

	if (on) {
		axi_gpio_write(sc->sc_axi, AXI_GPIO3, 3, 1);
		axi_gpio_write(sc->sc_axi, AXI_GPIO4, 9, 1);
		delay(200000);
		axi_gpio_write(sc->sc_axi, AXI_GPIO4, 10, 1);
	} else {
		axi_gpio_write(sc->sc_axi, AXI_GPIO4, 10, 0);
		delay(200000);
		axi_gpio_write(sc->sc_axi, AXI_GPIO4, 9, 0);
		axi_gpio_write(sc->sc_axi, AXI_GPIO3, 3, 0);
	}
#endif /* NETWALKER */
}


static int
ipu_err_intr(void *arg)
{
	struct ipu_softc *sc;

	sc = (struct ipu_softc *)arg;

	/* XXX: notyet */
	printf("%s:%d\n", __func__, __LINE__);

	return 1;
}

static int
ipu_sync_intr(void *arg)
{
	struct ipu_softc *sc;

	sc = (struct ipu_softc *)arg;

	/* XXX: notyet */
	printf("%s:%d\n", __func__, __LINE__);

	return 1;
}
